export function calculateRegularShape(options: {
  innerRadius?: number;
  outerRadius?: number;
  edgeLength?: number;
  edgeCount?: number;
  roundingMethod?: "grow" | "shrink" | "best";
}) {
  options = { ...options };

  if (typeof options.innerRadius === "number" && options.innerRadius <= 0) {
    throw new Error("Inner radius must be greater than 0");
  }
  if (typeof options.outerRadius === "number" && options.outerRadius <= 0) {
    throw new Error("Outer radius must be greater than 0");
  }
  if (typeof options.edgeLength === "number" && options.edgeLength <= 0) {
    throw new Error("Edge length must be greater than 0");
  }
  if (
    typeof options.edgeCount === "number" &&
    (options.edgeCount <= 2 || !Number.isInteger(options.edgeCount))
  ) {
    throw new Error("Edge count must be an integer greater than 2");
  }

  if (
    (typeof options.innerRadius === "number" ? 1 : 0) +
      (typeof options.outerRadius === "number" ? 1 : 0) +
      (typeof options.edgeLength === "number" ? 1 : 0) +
      (typeof options.edgeCount === "number" ? 1 : 0) !==
    2
  ) {
    throw new Error(
      "Only two of innerRadius/outerRadius, edgeLength, edgeCount can be defined"
    );
  }

  if (typeof options.roundingMethod !== "string") {
    options.roundingMethod = "best";
  }

  function normalizeEdgeCount(options: {
    innerRadius?: number;
    outerRadius?: number;
    edgeLength?: number;
    edgeCount?: number;
    roundingMethod?: "grow" | "shrink" | "best";
  }) {
    const intEdgeCount =
      options.roundingMethod === "grow"
        ? Math.ceil(options.edgeCount)
        : options.roundingMethod === "shrink"
        ? Math.floor(options.edgeCount)
        : Math.round(options.edgeCount);
    const scale = intEdgeCount / options.edgeCount;
    options.edgeLength *= scale;
    options.outerRadius *= scale;
    options.innerRadius *= scale;
    options.edgeCount = intEdgeCount;
  }

  if (options.edgeLength && options.edgeCount) {
    options.outerRadius =
      options.edgeLength / (2 * Math.sin(Math.PI / options.edgeCount));
    options.innerRadius =
      options.edgeLength / (2 * Math.tan(Math.PI / options.edgeCount));
  } else if (options.innerRadius && options.edgeCount) {
    options.outerRadius =
      options.innerRadius / Math.cos(Math.PI / options.edgeCount);
    options.edgeLength =
      2 * options.innerRadius * Math.sin(Math.PI / options.edgeCount);
  } else if (options.outerRadius && options.edgeCount) {
    options.innerRadius =
      options.outerRadius * Math.cos(Math.PI / options.edgeCount);
    options.edgeLength =
      2 * options.outerRadius * Math.sin(Math.PI / options.edgeCount);
  } else if (options.innerRadius && options.outerRadius) {
    options.edgeLength =
      2 *
      Math.sqrt(
        Math.pow(options.outerRadius, 2) - Math.pow(options.innerRadius, 2)
      );
    options.edgeCount =
      Math.PI / Math.asin(options.edgeLength / (2 * options.outerRadius));
    normalizeEdgeCount(options);
  } else if (options.innerRadius && options.edgeLength) {
    options.outerRadius = Math.sqrt(
      Math.pow(options.innerRadius, 2) + Math.pow(options.edgeLength / 2, 2)
    );
    options.edgeCount =
      Math.PI / Math.asin(options.edgeLength / (2 * options.outerRadius));
    normalizeEdgeCount(options);
  } else if (options.outerRadius && options.edgeLength) {
    options.innerRadius = Math.sqrt(
      Math.pow(options.outerRadius, 2) - Math.pow(options.edgeLength / 2, 2)
    );
    options.edgeCount =
      Math.PI / Math.asin(options.edgeLength / (2 * options.outerRadius));
    normalizeEdgeCount(options);
  }

  return options;
}
